///////////////////////////////////////////////////////////////////////////////
//
// Служебный модуль получения результата сравнения расширения, основной конфы и поставки
//
// (с) BIA Technologies, LLC
//
///////////////////////////////////////////////////////////////////////////////

// Сравнивает три конфигурации - Родитель, Поставка и Расширение
//
// Параметры: 
//   РодительПуть - Строка - Каталог конфигурации Родитель
//   ПоставкаПуть - Строка - Каталог конфигурации Поставка
//   РасширениеПуть - Строка - Каталог конфигурации Расширение
//   Лог - log-manager - Экземпляр класса логирования
//
// Возвращаемое значение:
//   Структура - результат сравнения
//
Функция ПолучитьРезультатСравнения(РодительПуть, ПоставкаПуть, РасширениеПуть, Лог) Экспорт

	РасширениеОписание = Новый Структура("Префикс, Типы");

	РасширениеОписание.Вставить("РодительПуть", РодительПуть);
	РасширениеОписание.Вставить("ПоставкаПуть", ПоставкаПуть);
	РасширениеОписание.Вставить("РасширениеПуть", РасширениеПуть);

	Парсер = Новый ПарсерМодулейРасширения(РасширениеПуть);

	ФайлКонфигурации = ОбъединитьПути(РасширениеПуть, "Configuration.xml");
	РасширениеОписание.Префикс = Парсер.ПрочитатьПрефиксРасширения(ФайлКонфигурации);

	// Получить переопределенные объекты (файлы xml и bsl)
	РасширениеОписание.Типы = Парсер.ПолучитьПереопределенныеОбъекты();
	Для каждого ТипРасширения Из РасширениеОписание.Типы Цикл

		Для каждого ОбъектРасширения Из ТипРасширения.Объекты Цикл

			Лог.Информация(ОбъектРасширения.Тип + " - " + ОбъектРасширения.Имя);
			Для каждого ФайлМодуля Из ОбъектРасширения.Модули Цикл

				МодульОбъекта = ФайлМодуля.ФайлМодуля;

				ФайлМодуля.Вставить("Методы", Парсер.ПрочитатьМетодыМодуля(МодульОбъекта, РасширениеОписание.Префикс));

				// читаем и раскладываем модуль родителя
				Парсер.ПрочитатьМетодыОсновнойКонфигурации(МодульОбъекта, ФайлМодуля, РодительПуть, "Родитель");

				// читаем и раскладываем модуль поставки
				Парсер.ПрочитатьМетодыОсновнойКонфигурации(МодульОбъекта, ФайлМодуля, ПоставкаПуть, "Поставка");

			КонецЦикла;

		КонецЦикла;
		
	КонецЦикла;

	Возврат РасширениеОписание;

КонецФункции
